"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.BookingCalendar = BookingCalendar;
var React = _interopRequireWildcard(require("react"));
var _classes = require("./classes");
var _classnames = _interopRequireDefault(require("classnames"));
var _common = require("./common");
var _BookingCalendarControls = require("./bookingCalendarControls/BookingCalendarControls");
var _utils = require("./utils");
var _BookingCalendarLane = require("./bookingCalendarLane/BookingCalendarLane");
var _moment = _interopRequireDefault(require("moment"));
var _bookingCalendarItem = require("./bookingCalendarItem");
var _BookingCalendarGrid = require("./bookingCalendarGrid/BookingCalendarGrid");
var _bookingCalendarLaneHeader = require("./bookingCalendarLaneHeader");
var _bookingCalendarLanesHeader = require("./bookingCalendarLanesHeader");
var _bookingCalendarDatePicker = require("./bookingCalendarDatePicker");
var _BookingCalendarTimeAxis = require("./bookingCalendarTimeAxis/BookingCalendarTimeAxis");
var _reactCoolDimensions = _interopRequireDefault(require("react-cool-dimensions"));
var _BookingCalendarSelection = _interopRequireDefault(require("./bookingCalendarSelection/BookingCalendarSelection"));
var _bookingCalendarLane = require("./bookingCalendarLane");
var _classes2 = require("./bookingCalendarSelection/classes");
var _bookingCalendarNowSection = require("./bookingCalendarNowSection");
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }
function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }
function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }
function _extends() { _extends = Object.assign ? Object.assign.bind() : function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }
const defaultStep = _moment.default.duration(1, 'day');
function BookingCalendar(props) {
  var _entry$target;
  const {
    getMoment = () => (0, _moment.default)(),
    bookings,
    className,
    laneContainerClassName,
    laneHeaderContainerClassName,
    lanesHeaderContainerClassName,
    lanesHeaderHeaderContainerClassName,
    controlsClasses,
    tableClassName,
    from,
    till,
    onRangeChange,
    step = defaultStep,
    showGrid = true,
    showHeader = true,
    showSelection = true,
    showNowSection = true,
    alwaysShowNowSections = false,
    onSelection,
    minSelectionSize = 10,
    selectionContent,
    showZoomAllButton = true,
    gridAvailableSteps = _common.defaultGridAvailableSteps,
    getMomentFormatFunction = _common.defaultGetMomentFormatFunction,
    getNewMomentFunction = _common.defaultGetNewMomentFunction,
    goalGridWidthPx = 60,
    minLanesCount,
    lanesSource,
    unmatchedLanesToFront = true,
    filterBookingsToZoom,
    zoomLevels,
    BookingCalendarControls = _BookingCalendarControls.BookingCalendarControls,
    BookingCalendarItem,
    BookingCalendarLane = _BookingCalendarLane.BookingCalendarLane,
    BookingCalendarRenderItem,
    BookingCalendarGrid = _BookingCalendarGrid.BookingCalendarGrid,
    BookingCalendarLaneHeader = _bookingCalendarLaneHeader.BookingCalendarLaneHeader,
    BookingCalendarLanesHeader = _bookingCalendarLanesHeader.BookingCalendarLanesHeader,
    BookingCalendarDatePicker = _bookingCalendarDatePicker.BookingCalendarDatePicker,
    BookingCalendarNowSection = _bookingCalendarNowSection.BookingCalendarNowSection,
    bookingCalendarTopLeftHeader,
    filteringButton,
    headersClickEnabled = true,
    onHeaderClick,
    children
  } = props;
  const {
    showTimeAxis = Boolean(bookings === null || bookings === void 0 ? void 0 : bookings.length) || Boolean(lanesSource === null || lanesSource === void 0 ? void 0 : lanesSource.length)
  } = props;
  const lanes = React.useMemo(() => (0, _utils.splitBookingsToLanes)(bookings, from, till, minLanesCount, lanesSource, unmatchedLanesToFront), [bookings, from, till, minLanesCount, lanesSource, unmatchedLanesToFront]);
  const {
    observe,
    entry
  } = (0, _reactCoolDimensions.default)();
  const width = (entry === null || entry === void 0 ? void 0 : (_entry$target = entry.target) === null || _entry$target === void 0 ? void 0 : _entry$target.scrollWidth) ?? 0;
  const {
    items: gridItems,
    bestStep
  } = React.useMemo(() => {
    var _from$clone;
    return (0, _utils.generateGridItems)(from, till, step, width, gridAvailableSteps, goalGridWidthPx, from === null || from === void 0 ? void 0 : (_from$clone = from.clone()) === null || _from$clone === void 0 ? void 0 : _from$clone.startOf('day'));
  }, [from, till, step, width, gridAvailableSteps, goalGridWidthPx]);
  const containerRef = React.useRef();
  const onSelected = React.useCallback((data, e) => {
    if (!from || !till || !width || !onRangeChange) {
      return;
    }
    const selectionLeft = Math.min(data.origin[0] - window.scrollX, data.target[0] - window.scrollX);
    const selectionRight = Math.max(data.origin[0] - window.scrollX, data.target[0] - window.scrollX);
    const selectionTop = Math.min(data.origin[1] - window.scrollY, data.target[1] - window.scrollY);
    const selectionBottom = Math.max(data.origin[1] - window.scrollY, data.target[1] - window.scrollY);
    const durationMs = till.valueOf() - from.valueOf();
    const toTimeSpace = num => num / (width || 1) * durationMs;
    const timeSpaceStart = from.clone().add(toTimeSpace(selectionLeft), 'ms');
    const timeSpaceEnd = from.clone().add(toTimeSpace(selectionRight), 'ms');
    if (onSelection) {
      var _containerRef$current, _containerRef$current2;
      const itemSelector = `.${_bookingCalendarLane.bookingCalendarLaneDefaultClasses.className}:not(.${_bookingCalendarLanesHeader.bookingCalendarLanesHeaderDefaultClasses.laneClassName}) > .${_bookingCalendarItem.bookingCalendarItemDefaultClasses.className}`;
      const allItems = Array.from(containerRef === null || containerRef === void 0 ? void 0 : (_containerRef$current = containerRef.current) === null || _containerRef$current === void 0 ? void 0 : _containerRef$current.querySelectorAll(itemSelector));
      const selectionAreaBB = containerRef === null || containerRef === void 0 ? void 0 : (_containerRef$current2 = containerRef.current) === null || _containerRef$current2 === void 0 ? void 0 : _containerRef$current2.querySelector(`.${_classes2.bookingCalendarSelectionDefaultClasses.root}`).getBoundingClientRect();
      const selectedItems = allItems.filter(item => {
        const itemBB = item.getBoundingClientRect();
        const itemLeft = itemBB.left - selectionAreaBB.left;
        const itemTop = itemBB.top - selectionAreaBB.top;
        const itemRight = itemBB.right - selectionAreaBB.left;
        const itemBottom = itemBB.bottom - selectionAreaBB.top;
        // check if the item rectangle is contained or intersects or overlaps with the selection rectangle
        return itemLeft < selectionRight && itemRight > selectionLeft && itemTop < selectionBottom && itemBottom > selectionTop;
      });
      const selectedIds = selectedItems.map(item => item.getAttribute('data-id'));
      onSelection(bookings === null || bookings === void 0 ? void 0 : bookings.filter(b => {
        var _b$id;
        return selectedIds.includes((_b$id = b.id) === null || _b$id === void 0 ? void 0 : _b$id.toString());
      }), timeSpaceStart, timeSpaceEnd, e);
    }
  }, [from, till, width, onRangeChange, onSelection, onSelection ? bookings : null]);
  const firstDataRowIndex = showHeader ? 2 : 1;
  return /*#__PURE__*/React.createElement("div", {
    className: (0, _classnames.default)(_classes.bookingCalendarDefaultClasses.className, className),
    ref: containerRef
  }, /*#__PURE__*/React.createElement(BookingCalendarControls, _extends({}, controlsClasses, {
    getMoment: getMoment,
    items: bookings,
    from: from,
    till: till,
    onRangeChange: onRangeChange,
    step: step,
    filterBookingsToZoom: filterBookingsToZoom,
    zoomLevels: zoomLevels,
    filteringButton: filteringButton,
    showZoomAllButton: showZoomAllButton,
    bookingCalendarDatePicker: /*#__PURE__*/React.createElement(React.Fragment, null, typeof BookingCalendarDatePicker === 'function' ? /*#__PURE__*/React.createElement(BookingCalendarDatePicker, {
      from: from,
      till: till,
      onRangeChange: onRangeChange,
      getNewMomentFunction: getNewMomentFunction
    }) : BookingCalendarDatePicker)
  })), /*#__PURE__*/React.createElement("div", {
    className: (0, _classnames.default)(_classes.bookingCalendarDefaultClasses.tableClassName, tableClassName)
  }, showGrid && /*#__PURE__*/React.createElement(BookingCalendarGrid, {
    firstDataRowIndex: firstDataRowIndex,
    items: gridItems,
    dataRowsCount: lanes.length,
    width: width,
    containerRef: showHeader ? undefined : observe
  }), showNowSection && /*#__PURE__*/React.createElement(BookingCalendarNowSection, {
    firstDataRowIndex: firstDataRowIndex,
    from: from,
    till: till,
    dataRowsCount: lanes.length,
    width: width,
    alwaysShowSections: alwaysShowNowSections
  }), showSelection && /*#__PURE__*/React.createElement(_BookingCalendarSelection.default, {
    firstDataRowIndex: firstDataRowIndex,
    dataRowsCount: lanes.length + (showTimeAxis ? 1 : 0),
    onSelected: onSelected,
    width: width,
    minSelectionSize: minSelectionSize
  }, selectionContent), showHeader && /*#__PURE__*/React.createElement(React.Fragment, null, /*#__PURE__*/React.createElement("div", {
    className: (0, _classnames.default)(_classes.bookingCalendarDefaultClasses.lanesHeaderHeaderContainerClassName, lanesHeaderHeaderContainerClassName)
  }, bookingCalendarTopLeftHeader), /*#__PURE__*/React.createElement("div", {
    className: (0, _classnames.default)(_classes.bookingCalendarDefaultClasses.lanesHeaderContainerClassName, lanesHeaderContainerClassName),
    ref: observe
  }, /*#__PURE__*/React.createElement(BookingCalendarLanesHeader, _extends({}, controlsClasses, {
    from: from,
    till: till,
    onRangeChange: headersClickEnabled ? onHeaderClick ?? onRangeChange : undefined,
    step: step
  })))), lanes.map((lane, laneIndex) => {
    const LaneBookingCalendarLaneHeader = lane.BookingCalendarLaneHeader ?? BookingCalendarLaneHeader;
    const LaneBookingCalendarLane = lane.BookingCalendarLane ?? BookingCalendarLane;
    return /*#__PURE__*/React.createElement(React.Fragment, {
      key: laneIndex
    }, /*#__PURE__*/React.createElement("div", {
      className: (0, _classnames.default)(_classes.bookingCalendarDefaultClasses.laneHeaderContainerClassName, laneHeaderContainerClassName, lane.rowClassName, {
        [`${_classes.bookingCalendarDefaultClasses.laneHeaderContainerClassName}--last`]: laneIndex === lanes.length - 1,
        [`${_classes.bookingCalendarDefaultClasses.laneHeaderContainerClassName}--first`]: laneIndex === 0
      })
    }, /*#__PURE__*/React.createElement(LaneBookingCalendarLaneHeader, {
      laneKey: lane.laneKey ?? laneIndex,
      data: lane.data
    })), /*#__PURE__*/React.createElement("div", {
      className: (0, _classnames.default)(_classes.bookingCalendarDefaultClasses.laneContainerClassName, laneContainerClassName, lane.rowClassName, {
        [`${_classes.bookingCalendarDefaultClasses.laneContainerClassName}--last`]: laneIndex === lanes.length - 1,
        [`${_classes.bookingCalendarDefaultClasses.laneContainerClassName}--first`]: laneIndex === 0
      })
    }, /*#__PURE__*/React.createElement(LaneBookingCalendarLane, {
      laneIndex: laneIndex,
      items: lane.items,
      from: from,
      till: till,
      BookingCalendarItem: lane.BookingCalendarItem ?? BookingCalendarItem,
      BookingCalendarRenderItem: lane.BookingCalendarRenderItem ?? BookingCalendarRenderItem,
      step: step
    })));
  }), /*#__PURE__*/React.createElement("div", null), showTimeAxis && /*#__PURE__*/React.createElement("div", {
    ref: showGrid || showGrid ? undefined : observe
  }, /*#__PURE__*/React.createElement(_BookingCalendarTimeAxis.BookingCalendarTimeAxis, {
    items: gridItems,
    bestStep: bestStep,
    getMomentFormatFunction: getMomentFormatFunction
  }))), children);
}
//# sourceMappingURL=BookingCalendar.js.map