"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.BookingCalendarDatePicker = BookingCalendarDatePicker;
var React = _interopRequireWildcard(require("react"));
var _classnames = _interopRequireDefault(require("classnames"));
var _classes = require("./classes");
var _common = require("../common");
var _Button = require("../../button/Button");
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }
function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }
function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }
var EventIcon = function EventIcon(props) {
  return /*#__PURE__*/React.createElement("svg", props, /*#__PURE__*/React.createElement("path", {
    d: "M17 12h-5v5h5v-5zM16 1v2H8V1H6v2H5c-1.11 0-1.99.9-1.99 2L3 19a2 2 0 0 0 2 2h14c1.1 0 2-.9 2-2V5c0-1.1-.9-2-2-2h-1V1h-2zm3 18H5V8h14v11z"
  }));
};
EventIcon.defaultProps = {
  xmlns: "http://www.w3.org/2000/svg",
  width: "24",
  height: "24",
  viewBox: "0 0 24 24"
};
try {
  var DatePicker = require('react-datepicker').default;
} catch {
  DatePicker = undefined;
}
// import DatePicker from 'react-datepicker';

function BookingCalendarDatePicker(props) {
  const {
    className,
    buttonClassName,
    from,
    till,
    onRangeChange,
    getNewMomentFunction = _common.defaultGetNewMomentFunction
  } = props;
  const [shownModal, setShownModal] = React.useState();
  const calendarRef = React.useRef();
  const onButtonClick = React.useCallback(() => {
    setShownModal('start');
    calendarRef.current.setOpen(true);
  }, []);
  return /*#__PURE__*/React.createElement("div", {
    className: (0, _classnames.default)(_classes.bookingCalendarDatePickerDefaultClasses.className, className)
  }, /*#__PURE__*/React.createElement(_Button.Button, {
    className: (0, _classnames.default)(_classes.bookingCalendarDatePickerDefaultClasses.buttonClassName, buttonClassName),
    noShadow: true,
    onClick: onButtonClick
  }, /*#__PURE__*/React.createElement(EventIcon, null), " ", from === null || from === void 0 ? void 0 : from.format('MMMM, YYYY')), /*#__PURE__*/React.createElement(DatePicker, {
    ref: calendarRef,
    customInput: /*#__PURE__*/React.createElement("span", null),
    selected: from === null || from === void 0 ? void 0 : from.toDate()
    // selectsRange={true}
    ,
    selectsStart: shownModal !== 'end',
    selectsEnd: shownModal === 'end',
    startDate: from === null || from === void 0 ? void 0 : from.toDate(),
    endDate: till === null || till === void 0 ? void 0 : till.toDate(),
    shouldCloseOnSelect: shownModal === 'end'
    //dateFormat={DAY_FORMAT}
    ,
    onChange: dt => {
      if (shownModal !== 'end') {
        const dtm = getNewMomentFunction(dt);
        let validTill = till.clone();
        if (dtm.isAfter(till)) {
          validTill = dtm.clone().add(1, 'day');
        }
        onRangeChange({
          from: dtm,
          till: validTill
        });
        setShownModal('end');
      } else {
        let dtm = getNewMomentFunction(dt).startOf('day').add(1, 'day');
        let validFrom = from.clone();
        if (dtm.isBefore(from)) {
          validFrom = dtm.clone().add(-1, 'day');
        }
        onRangeChange({
          from: validFrom,
          till: dtm
        });
        setShownModal(null);
      }
    },
    withPortal: true
  }));
}
//# sourceMappingURL=BookingCalendarDatePicker.js.map