/// <reference types="react" />
import moment, { Moment, Duration } from 'moment';
import { BookingCalendarItemProps } from './bookingCalendarItem';
import { BookingCalendarLaneProps } from './bookingCalendarLane';
import { BookingCalendarLaneHeaderProps } from './bookingCalendarLaneHeader';
import { BookingCalendarRenderItemProps } from './bookingCalendarRenderItem';
import { BookingCalendarItemT } from './common';
export declare function groupBy<T>(items: T[], iterator: (item: T) => number | string): {
    [key: string | number]: T[];
};
export interface LaneSourceData<T extends BookingCalendarItemT, TLaneData> {
    laneKey: number;
    data?: TLaneData;
    rowClassName?: string;
    BookingCalendarItem?: React.ComponentType<BookingCalendarItemProps<T>>;
    BookingCalendarRenderItem?: React.ComponentType<BookingCalendarRenderItemProps<T>>;
    BookingCalendarLaneHeader?: React.ComponentType<BookingCalendarLaneHeaderProps<TLaneData>>;
    BookingCalendarLane?: React.ComponentType<BookingCalendarLaneProps<T>>;
}
export interface LaneData<T extends BookingCalendarItemT, TLaneData> extends LaneSourceData<T, TLaneData> {
    items: T[];
}
export declare function splitBookingsToLanes<T extends BookingCalendarItemT, TLaneData>(bookings: T[], from: Moment, till: Moment, minLanesCount: number, lanesSource?: LaneSourceData<T, TLaneData>[], unmatchedToFront?: boolean): LaneData<T, TLaneData>[];
type Picked = Pick<BookingCalendarItemT, 'from' | 'till'>;
export declare function itemsOverlap(a: Picked, b: Picked): boolean;
export declare const getItemPositionStyle: (marginStart: number, width: number) => import("react").CSSProperties;
export declare function calculateItemsDimensions<T extends BookingCalendarItemT>(items: T[], from: Moment, till: Moment): {
    item: T;
    start: number;
    realStart: number;
    realFrom: moment.Moment;
    startIsCut: boolean;
    width: number;
    end: number;
    realEnd: number;
    realTill: moment.Moment;
    endIsCut: boolean;
    marginStart: number;
}[];
export declare const generateControlItems: (from: Moment, till: Moment, step: Duration, startOfStep: Moment) => {
    from: moment.Moment;
    till: moment.Moment;
    id: number;
}[];
export interface GridItem {
    left: number;
    date: Moment;
    stepProportion: number;
    opacity: number;
}
export declare const generateGridItems: (from: Moment, till: Moment, step: Duration, containerWidthPx: number, availableSteps: Duration[], goalGridWidthPx: number, startOfStep: Moment) => {
    items: GridItem[];
    bestStep: Duration;
};
export {};
