"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.calculateItemsDimensions = calculateItemsDimensions;
exports.getItemPositionStyle = exports.generateGridItems = exports.generateControlItems = void 0;
exports.groupBy = groupBy;
exports.itemsOverlap = itemsOverlap;
exports.splitBookingsToLanes = splitBookingsToLanes;
var _moment = _interopRequireDefault(require("moment"));
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }
function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); enumerableOnly && (symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; })), keys.push.apply(keys, symbols); } return keys; }
function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = null != arguments[i] ? arguments[i] : {}; i % 2 ? ownKeys(Object(source), !0).forEach(function (key) { _defineProperty(target, key, source[key]); }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)) : ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } return target; }
function _defineProperty(obj, key, value) { key = _toPropertyKey(key); if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }
function _toPropertyKey(arg) { var key = _toPrimitive(arg, "string"); return typeof key === "symbol" ? key : String(key); }
function _toPrimitive(input, hint) { if (typeof input !== "object" || input === null) return input; var prim = input[Symbol.toPrimitive]; if (prim !== undefined) { var res = prim.call(input, hint || "default"); if (typeof res !== "object") return res; throw new TypeError("@@toPrimitive must return a primitive value."); } return (hint === "string" ? String : Number)(input); }
function groupBy(items, iterator) {
  return items.reduce((obj, item) => {
    const key = iterator(item);
    if (!obj.hasOwnProperty(key)) {
      obj[key] = [];
    }
    obj[key].push(item);
    return obj;
  }, {});
}
function splitBookingsToLanes(bookings, from, till, minLanesCount) {
  let lanesSource = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : [];
  let unmatchedToFront = arguments.length > 5 && arguments[5] !== undefined ? arguments[5] : true;
  if (!bookings) {
    return [];
  }
  let lanes = [];
  if (lanesSource !== null && lanesSource !== void 0 && lanesSource.length) {
    lanes = lanesSource.map(source => _objectSpread(_objectSpread({}, source), {}, {
      items: []
    }));
  }
  const bookingsWithoutLaneKey = bookings.filter(a => a.laneKey === undefined);
  const bookingsWithLaneKey = bookings.filter(a => a.laneKey !== undefined);
  if (bookingsWithLaneKey.length) {
    const grouped = groupBy(bookingsWithLaneKey.map((booking, originalIndex) => _objectSpread(_objectSpread({}, booking), {}, {
      originalIndex
    })), a => a.laneKey);
    lanes = Object.keys(grouped).map(key => _objectSpread({
      laneKey: Number(key),
      items: grouped[key]
    }, lanesSource === null || lanesSource === void 0 ? void 0 : lanesSource.find(a => a.laneKey === Number(key))));
    const missingLanes = lanesSource === null || lanesSource === void 0 ? void 0 : lanesSource.filter(a => !lanes.some(l => l.laneKey === a.laneKey));
    lanes = lanes.concat(missingLanes.map(a => _objectSpread(_objectSpread({}, a), {}, {
      items: []
    })));
  }
  if (bookingsWithoutLaneKey.length) {
    let remainingBookings = bookingsWithoutLaneKey.map((booking, originalIndex) => _objectSpread(_objectSpread({}, booking), {}, {
      originalIndex
    })).sort((a, b) => a.from.valueOf() - b.from.valueOf());
    while (remainingBookings.length > 0) {
      let min = 99999999999999;
      let bookingIndex = -1;
      let laneIndex = -1;
      lanes.forEach((lane, index) => {
        var _lane$items;
        if (lanesSource !== null && lanesSource !== void 0 && lanesSource.some(l => l.laneKey === lane.laneKey)) {
          return;
        }
        const lastTill = ((_lane$items = lane.items[lane.items.length - 1]) === null || _lane$items === void 0 ? void 0 : _lane$items.till) ?? from;
        const distances = remainingBookings.map(booking => {
          const dist = booking.from.diff(lastTill);
          if (lane.items.length === 0) {
            return Math.abs(dist);
          }
          return dist;
        });
        const _min = Math.min(...distances.filter(a => a >= 0));
        if (_min < min) {
          bookingIndex = distances.indexOf(_min);
          laneIndex = index;
        }
      });
      if (bookingIndex >= 0 && laneIndex >= 0) {
        lanes[laneIndex].items = lanes[laneIndex].items.concat(_objectSpread({}, remainingBookings[bookingIndex]));
        remainingBookings.splice(bookingIndex, 1);
      } else {
        lanes = lanes.concat({
          items: [_objectSpread({}, remainingBookings[0])],
          laneKey: lanes.length,
          data: undefined
        });
        remainingBookings.shift();
      }
    }
    // Naive solution that is much less computationally intensive but yields more lanes than necessary
    /*bookings.forEach((booking, originalIndex) => {
    let found = false;
    lanes.forEach((lane, index) => {
      if (!lane.some(i => itemsOverlap(booking, i))) {
        lanes[index] = lanes[index].concat({ ...booking, originalIndex });
        found = true;
      }
    });
    if (!found) {
      lanes = lanes.concat([[{ ...booking, originalIndex }]]);
    }
    });*/
  }

  for (let index = 0; index < lanes.length; index++) {
    const laneItems = lanes[index].items;
    const normal = laneItems.filter(a => a.till.valueOf() < till.valueOf());
    normal.sort((a, b) => a.from.valueOf() - b.from.valueOf());
    const atEnd = laneItems.filter(a => a.till.valueOf() >= till.valueOf());
    atEnd.sort((a, b) => a.from.valueOf() - b.from.valueOf());
    lanes[index].items = [...normal, ...atEnd];
  }
  if (lanes.length < minLanesCount) {
    lanes = lanes.concat(new Array(minLanesCount - lanes.length).fill(null).map((_, index) => ({
      items: [],
      laneKey: index + lanes.length,
      data: null
    })));
  }
  const findIndex = (arr, laneKey) => {
    const unmatchedVal = unmatchedToFront ? Number.MIN_SAFE_INTEGER : Number.MAX_SAFE_INTEGER;
    if (laneKey === undefined || laneKey === null) {
      return unmatchedVal;
    }
    const index = arr.findIndex(item => item.laneKey === laneKey);
    return index === -1 ? unmatchedVal : index;
  };
  const sortArrayByLaneKeys = (referenceArray, arrayToSort) => {
    return [...arrayToSort].sort((a, b) => {
      const indexA = findIndex(referenceArray, a.laneKey);
      const indexB = findIndex(referenceArray, b.laneKey);
      return indexA - indexB;
    });
  };
  lanes = sortArrayByLaneKeys(lanesSource, lanes);
  return lanes;
}
function itemsOverlap(a, b) {
  if (!(a !== null && a !== void 0 && a.from) || !(a !== null && a !== void 0 && a.till) || !(b !== null && b !== void 0 && b.from) || !(b !== null && b !== void 0 && b.till)) {
    return false;
  }
  if (b.till.isBefore(a.from)) {
    return false;
  }
  if (b.from.isAfter(a.till)) {
    return false;
  }
  return true;
}
const getItemPositionStyle = (marginStart, width) => ({
  marginLeft: `${marginStart * 100}%`,
  width: `${width * 100}%`
});
exports.getItemPositionStyle = getItemPositionStyle;
function calculateItemsDimensions(items, from, till) {
  let lastEnd = 0;
  const widthMs = till.valueOf() - from.valueOf();
  const startMs = from.valueOf();
  const endMs = till.valueOf();
  return widthMs > 0 ? items === null || items === void 0 ? void 0 : items.map(item => {
    const startIsCut = item.from.valueOf() < startMs;
    const realStart = (item.from.valueOf() - startMs) / widthMs;
    const realFrom = startIsCut ? (0, _moment.default)(startMs) : (0, _moment.default)(item.from);
    const start = startIsCut ? 0 : realStart;
    const endIsCut = item.till.valueOf() > endMs;
    const realEnd = (item.till.valueOf() - startMs) / widthMs;
    const end = endIsCut ? (endMs - startMs) / widthMs : realEnd;
    const realTill = endIsCut ? (0, _moment.default)(endMs) : (0, _moment.default)(item.till);
    const marginStart = start - lastEnd;
    const width = end - start;
    if (end > 0 && start < 1) {
      lastEnd = Math.max(Math.min(realEnd, 1), 0);
    }
    return {
      item,
      start,
      realStart,
      realFrom,
      startIsCut,
      width,
      end,
      realEnd,
      realTill,
      endIsCut,
      marginStart
    };
  }) : [];
}
const generateControlItems = (from, till, step, startOfStep) => {
  if (!from || !till || !step) {
    return [];
  }
  let steps = Math.max(Math.ceil((till.valueOf() - from.valueOf()) / step.asMilliseconds()), 2);
  let subtract = (startOfStep.valueOf() - from.valueOf()) % step.asMilliseconds();
  if (subtract > 0) {
    subtract -= step.asMilliseconds();
  }
  if (till.valueOf() - from.valueOf() > step.asMilliseconds()) {
    steps += 1;
  }
  return new Array(steps).fill(0).map((_, index) => ({
    from: from.clone().add(subtract).add(step.asMilliseconds() * index),
    till: from.clone().add(subtract).add(step.asMilliseconds() * (index + 1)),
    id: index
  }));
};
exports.generateControlItems = generateControlItems;
const generateGridItems = (from, till, step, containerWidthPx, availableSteps, goalGridWidthPx, startOfStep) => {
  if (!from || !till || !containerWidthPx || !(availableSteps !== null && availableSteps !== void 0 && availableSteps.length) || !goalGridWidthPx || !step || !startOfStep) {
    return {
      bestStep: availableSteps === null || availableSteps === void 0 ? void 0 : availableSteps[0],
      items: []
    };
  }
  const stepMs = step.asMilliseconds();
  const fromMs = from.valueOf();
  const tillMs = till.valueOf();
  const timeLengthMs = tillMs - fromMs;
  const approximateStepsCount = containerWidthPx / goalGridWidthPx;
  const approximateStepMs = timeLengthMs / approximateStepsCount;
  const availableStepsDistancesMs = availableSteps.map(a => Math.abs(a.asMilliseconds() - approximateStepMs));
  const smallestDistance = Math.min(...availableStepsDistancesMs);
  const bestStepIndex = availableStepsDistancesMs.findIndex(a => a === smallestDistance);
  const bestStep = availableSteps[bestStepIndex];
  const subtract = (fromMs + from.utcOffset() * 60 * 1000) % bestStep.asMilliseconds();
  let startMs = fromMs - subtract;
  let steps = [];
  while (startMs <= tillMs) {
    steps = steps.concat(from.clone().add(startMs - fromMs, 'ms'));
    startMs += bestStep.asMilliseconds();
  }
  return {
    items: steps.filter(a => a.valueOf() > fromMs && a.valueOf() < tillMs).map(date => {
      const stepProportion = (date.valueOf() + from.utcOffset() * 60 * 1000) % stepMs / stepMs;
      let numberOfRoots = 0;
      let current = stepProportion;
      let currentTop = 1;
      let tolerance = 0.001;
      while (current > tolerance && current < currentTop - tolerance) {
        numberOfRoots += 1;
        currentTop = 1 / Math.pow(2, numberOfRoots);
        if (current > currentTop) {
          current -= currentTop;
        }
      }
      return {
        left: (date.valueOf() - fromMs) / timeLengthMs,
        date,
        stepProportion,
        opacity: 1 / (numberOfRoots || 1)
      };
    }),
    bestStep
  };
};
exports.generateGridItems = generateGridItems;
//# sourceMappingURL=utils.js.map