import * as React from 'react';
import type { TippyProps } from '@tippy.js/react';
import { ThemeContextProps } from '../themeProvider/ThemeContext';
export type ButtonTypes = 'primary' | 'success' | 'error' | 'warning' | 'info' | 'gray' | 'white' | 'black' | 'none';
export type ButtonComponentProps = {
    id?: string;
    onClick: (e: React.MouseEvent<HTMLElement>) => void;
    className: string;
    buttonProps: React.ButtonHTMLAttributes<HTMLButtonElement>;
    children: React.ReactNode;
    style: React.CSSProperties;
};
export type ButtonProps = React.PropsWithChildren<{
    id?: string;
    onClick?: (e: React.MouseEvent<HTMLButtonElement>) => void;
    style?: React.CSSProperties;
    className?: string;
    disabled?: boolean;
    type?: ButtonTypes;
    circular?: boolean;
    noRipples?: boolean;
    small?: boolean;
    icon?: boolean;
    outlined?: boolean;
    disableAfterClickMs?: number;
    disableAfterClick?: boolean;
    buttonProps?: React.ButtonHTMLAttributes<HTMLButtonElement>;
    text?: boolean;
    hero?: boolean;
    noShadow?: boolean;
    blank?: boolean;
    unobtrusive?: boolean;
    dropdown?: boolean;
    Component?: React.FC<ButtonComponentProps>;
    tooltip?: JSX.Element | string;
    disableTooltip?: boolean;
    tooltipProps?: TippyProps;
    preventsDefault?: boolean;
} & ThemeContextProps>;
export interface ButtonState {
    disabledAfterClick: boolean;
}
export declare class Button extends React.PureComponent<ButtonProps, ButtonState> {
    static defaultProps: ButtonProps;
    private preventMultipleClickTimer;
    constructor(props: ButtonProps);
    render(): React.JSX.Element;
    componentWillUnmount(): void;
    private handleClick;
    private getButtonClassName;
}
declare const _default: React.ForwardRefExoticComponent<{
    id?: string;
    onClick?: (e: React.MouseEvent<HTMLButtonElement, MouseEvent>) => void;
    style?: React.CSSProperties;
    className?: string;
    disabled?: boolean;
    type?: ButtonTypes;
    circular?: boolean;
    noRipples?: boolean;
    small?: boolean;
    icon?: boolean;
    outlined?: boolean;
    disableAfterClickMs?: number;
    disableAfterClick?: boolean;
    buttonProps?: React.ButtonHTMLAttributes<HTMLButtonElement>;
    text?: boolean;
    hero?: boolean;
    noShadow?: boolean;
    blank?: boolean;
    unobtrusive?: boolean;
    dropdown?: boolean;
    Component?: React.FC<ButtonComponentProps>;
    tooltip?: string | JSX.Element;
    disableTooltip?: boolean;
    tooltipProps?: TippyProps;
    preventsDefault?: boolean;
} & ThemeContextProps & {
    children?: React.ReactNode;
} & React.RefAttributes<Button>>;
export default _default;
