"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = exports.Dropdown = void 0;
var React = _interopRequireWildcard(require("react"));
var _Collapse = _interopRequireDefault(require("@mui/material/Collapse"));
var _withThemeContext = require("../themeProvider/withThemeContext");
var _classnames = _interopRequireDefault(require("classnames"));
var _ClickAwayListener = _interopRequireDefault(require("@mui/material/ClickAwayListener"));
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }
function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }
function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }
function _extends() { _extends = Object.assign ? Object.assign.bind() : function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); } // Libs
const Dropdown = props => {
  const {
    shouldHandleClick = true,
    WrapperTag = 'div',
    notificationCount = 0,
    showArrow = true,
    inline = true,
    onClick,
    onShow,
    onHide,
    disabled,
    className,
    headerClassName,
    header,
    id,
    collapseProps,
    children,
    submenuClassName
  } = props;
  const [isDropdownVisible, setIsDropdownVisible] = React.useState(false);
  const hideNavigation = React.useCallback(() => {
    setIsDropdownVisible(false);
    onHide === null || onHide === void 0 ? void 0 : onHide();
  }, [onHide]);
  const showNavigation = React.useCallback(() => {
    setIsDropdownVisible(true);
    onShow === null || onShow === void 0 ? void 0 : onShow();
  }, [onShow]);
  const onClickAway = React.useCallback(e => {
    hideNavigation();
    e.stopPropagation();
    e.preventDefault();
  }, [hideNavigation]);
  const handleClick = React.useCallback(e => {
    if (shouldHandleClick && !disabled) {
      if (!isDropdownVisible) {
        showNavigation();
      } else {
        hideNavigation();
      }
      onClick === null || onClick === void 0 ? void 0 : onClick(e, isDropdownVisible);
    }
  }, [isDropdownVisible, onClick]);
  const containerClassName = (0, _classnames.default)('guestbell__dropdown', !isDropdownVisible ? 'guestbell__dropdown--closed' : 'guestbell__dropdown--opened', {
    ['guestbell__dropdown--disabled']: disabled
  }, {
    ['guestbell__dropdown--inline']: inline
  }, className);
  const headerClassNameAll = (0, _classnames.default)(['guestbell__dropdown-toggle', {
    ['guestbell__dropdown-toggle__arrow--hidden']: !showArrow
  }, {
    ['guestbell__dropdown-toggle--disabled']: disabled
  }, headerClassName]);
  return /*#__PURE__*/React.createElement(_ClickAwayListener.default, {
    onClickAway: onClickAway,
    mouseEvent: isDropdownVisible ? 'onMouseDown' : false,
    touchEvent: isDropdownVisible ? 'onTouchStart' : false
  }, /*#__PURE__*/React.createElement(WrapperTag, {
    id: id ?? null,
    className: containerClassName
  }, /*#__PURE__*/React.createElement("div", {
    role: "button",
    className: headerClassNameAll,
    onClick: handleClick
  }, header, notificationCount > 0 && /*#__PURE__*/React.createElement("span", {
    className: "guestbell__label-count"
  }, notificationCount)), /*#__PURE__*/React.createElement("div", {
    className: 'guestbell__dropdown-menu__container'
  }, /*#__PURE__*/React.createElement(_Collapse.default, _extends({}, collapseProps, {
    in: isDropdownVisible
  }), /*#__PURE__*/React.createElement("ul", {
    className: (0, _classnames.default)('guestbell__dropdown-menu', submenuClassName)
  }, children)))));
};
exports.Dropdown = Dropdown;
var _default = (0, _withThemeContext.withThemeContext)(Dropdown);
exports.default = _default;
//# sourceMappingURL=Dropdown.js.map