"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = exports.DynamicSubmitRaw = exports.DynamicSubmitMode = exports.DynamicSubmit = void 0;
var React = _interopRequireWildcard(require("react"));
var _classnames = _interopRequireDefault(require("classnames"));
var _withFormContext = require("../form/withFormContext");
var _Submit = require("./../submit/Submit");
var _withThemeContext = require("../themeProvider/withThemeContext");
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }
function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }
function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }
function _extends() { _extends = Object.assign ? Object.assign.bind() : function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); } // Libs
// Misc
let DynamicSubmitMode = /*#__PURE__*/function (DynamicSubmitMode) {
  DynamicSubmitMode[DynamicSubmitMode["Normal"] = 0] = "Normal";
  DynamicSubmitMode[DynamicSubmitMode["Submitting"] = 1] = "Submitting";
  DynamicSubmitMode[DynamicSubmitMode["Error"] = 2] = "Error";
  DynamicSubmitMode[DynamicSubmitMode["Success"] = 3] = "Success";
  return DynamicSubmitMode;
}({});
exports.DynamicSubmitMode = DynamicSubmitMode;
class DynamicSubmitRaw extends React.PureComponent {
  constructor(props) {
    super(props);
    this.state = {
      buttonState: DynamicSubmitMode.Normal
    };
    this.handleClick = this.handleClick.bind(this);
    this.submitting = this.submitting.bind(this);
    this.error = this.error.bind(this);
    this.success = this.success.bind(this);
    this.reset = this.reset.bind(this);
  }
  render() {
    const buttonClassName = (0, _classnames.default)(this.props.className, this.renderClassName());
    return /*#__PURE__*/React.createElement(_Submit.SubmitRaw, _extends({}, this.props, {
      className: buttonClassName,
      onClick: this.handleClick
    }), this.renderChildren());
  }
  handleClick(e) {
    e.preventDefault();
    this.props.submitDisablesInputs && this.props.formContext.disableComponents();
    this.props.onClick && this.props.onClick(e, this.submitting, this.error, this.success, this.reset);
  }
  submitting() {
    this.setState({
      buttonState: DynamicSubmitMode.Submitting
    });
  }
  error() {
    this.setState({
      buttonState: DynamicSubmitMode.Error
    });
  }
  success() {
    this.setState({
      buttonState: DynamicSubmitMode.Success
    });
  }
  reset() {
    this.props.resetEnablesInputs && this.props.formContext.enableComponents();
    this.setState({
      buttonState: DynamicSubmitMode.Normal
    });
  }
  renderChildren() {
    switch (this.state.buttonState) {
      case DynamicSubmitMode.Error:
        return this.props.errorChildren ? this.props.errorChildren : this.props.children;
      case DynamicSubmitMode.Submitting:
        return this.props.submittingChildren ? this.props.submittingChildren : this.props.children;
      case DynamicSubmitMode.Success:
        return this.props.successChildren ? this.props.successChildren : this.props.children;
      default:
        return this.props.children;
    }
  }
  renderClassName() {
    switch (this.state.buttonState) {
      case DynamicSubmitMode.Error:
        return this.props.errorClassName ? this.props.errorClassName : '';
      case DynamicSubmitMode.Submitting:
        return this.props.submittingClassName ? this.props.submittingClassName : '';
      case DynamicSubmitMode.Success:
        return this.props.successClassName ? this.props.successClassName : '';
      default:
        return this.props.normalClassName ? this.props.normalClassName : '';
    }
  }
}
exports.DynamicSubmitRaw = DynamicSubmitRaw;
DynamicSubmitRaw.defaultProps = Object.assign({}, {
  validateForm: true,
  submitDisablesInputs: true,
  resetEnablesInputs: true
});
const DynamicSubmit = (0, _withThemeContext.withThemeContext)((0, _withFormContext.withFormContext)(DynamicSubmitRaw), 'dynamicSubmit');
exports.DynamicSubmit = DynamicSubmit;
var _default = DynamicSubmit;
exports.default = _default;
//# sourceMappingURL=DynamicSubmit.js.map