import * as React from 'react';
import { ButtonProps } from '../button/Button';
import { OmitInputHeaderContext, InputHeaderContextProps, InputHeaderContextState } from '../inputHeader/InputHeaderContext';
import type { TippyProps } from '@tippy.js/react';
import { ThemeContextProps } from '../themeProvider/ThemeContext';
export type InputHeaderRawProps = React.PropsWithChildren<{
    onExpanded?: () => void;
    onCollapsed?: () => void;
    onChange?: (collapsed: boolean) => void;
    className?: string;
    title?: string | JSX.Element;
    icon?: string | JSX.Element;
    subTitle?: string | JSX.Element;
    mainButton?: JSX.Element;
    extraButtons?: JSX.Element[];
    collapsible?: boolean;
    collapsed?: boolean;
    collapsedDefault?: boolean;
    ignoreContext?: boolean;
    showExpandAll?: boolean;
    type?: 'hero' | 'standard' | 'small';
    noBg?: boolean;
    headerClassName?: string;
    contentClassName?: string;
    extraButtonsButtonProps?: ButtonProps;
    collapseButtonsButtonProps?: ButtonProps;
    shouldToggleCollapseOnHeaderClick?: boolean;
    mountOnEnter?: boolean;
    unmountOnExit?: boolean;
    tooltip?: string | JSX.Element;
    tooltipProps?: TippyProps;
    stopsPropagation?: boolean;
    ButtonsContainer?: React.ComponentType<{
        className: string;
        onClick: (e: React.MouseEvent<HTMLElement>) => void;
    }> | 'div';
} & InputHeaderContextProps & ThemeContextProps>;
export type InputHeaderProps = OmitInputHeaderContext<InputHeaderRawProps>;
export interface InputHeaderApi {
    expand: () => void;
    collapse: () => void;
    toggle: () => void;
}
export interface InputHeaderState {
    collapsed: boolean;
    inputHeaderContext: InputHeaderContextState;
}
export declare class InputHeaderRaw extends React.PureComponent<InputHeaderRawProps, InputHeaderState> implements InputHeaderApi {
    static defaultProps: InputHeaderProps;
    componentId: string;
    constructor(props: InputHeaderRawProps);
    componentWillUnmount(): void;
    componentDidMount(): void;
    expand(): void;
    collapse(): void;
    toggle(): void;
    render(): React.JSX.Element;
    private toggleClick;
    private registerInputHeader;
    private unregisterInputHeader;
    private getTypeClass;
    private mainButtonClick;
    private renderExtraButtons;
    private renderCollapseExpandAll;
    private containerClick;
    private expandAllClick;
    private collapseAllClick;
    private renderTitle;
    private renderTooltip;
}
export declare const InputHeader: React.ForwardRefExoticComponent<{
    onExpanded?: () => void;
    onCollapsed?: () => void;
    onChange?: (collapsed: boolean) => void;
    className?: string;
    title?: string | JSX.Element;
    icon?: string | JSX.Element;
    subTitle?: string | JSX.Element;
    mainButton?: JSX.Element;
    extraButtons?: JSX.Element[];
    collapsible?: boolean;
    collapsed?: boolean;
    collapsedDefault?: boolean;
    ignoreContext?: boolean;
    showExpandAll?: boolean;
    type?: 'hero' | 'standard' | 'small';
    noBg?: boolean;
    headerClassName?: string;
    contentClassName?: string;
    extraButtonsButtonProps?: ButtonProps;
    collapseButtonsButtonProps?: ButtonProps;
    shouldToggleCollapseOnHeaderClick?: boolean;
    mountOnEnter?: boolean;
    unmountOnExit?: boolean;
    tooltip?: string | JSX.Element;
    tooltipProps?: TippyProps;
    stopsPropagation?: boolean;
    ButtonsContainer?: "div" | React.ComponentType<{
        className: string;
        onClick: (e: React.MouseEvent<HTMLElement>) => void;
    }>;
} & InputHeaderContextProps & ThemeContextProps & {
    children?: React.ReactNode;
} & React.RefAttributes<InputHeaderRaw>>;
export default InputHeader;
