"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.defaultMoneyTranslations = exports.default = exports.MoneyRaw = exports.Money = void 0;
var React = _interopRequireWildcard(require("react"));
var _Select = require("../select/Select");
var _Text = require("../text/Text");
var _InputGroup = _interopRequireDefault(require("../inputGroup/InputGroup"));
var _BaseInput = require("../base/input/BaseInput");
var _Button = require("../button/Button");
var _withFormContext = require("../form/withFormContext");
var _withThemeContext = require("../themeProvider/withThemeContext");
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }
function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }
function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }
function _extends() { _extends = Object.assign ? Object.assign.bind() : function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }
function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); enumerableOnly && (symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; })), keys.push.apply(keys, symbols); } return keys; }
function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = null != arguments[i] ? arguments[i] : {}; i % 2 ? ownKeys(Object(source), !0).forEach(function (key) { _defineProperty(target, key, source[key]); }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)) : ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } return target; }
function _defineProperty(obj, key, value) { key = _toPropertyKey(key); if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }
function _toPropertyKey(arg) { var key = _toPrimitive(arg, "string"); return typeof key === "symbol" ? key : String(key); }
function _toPrimitive(input, hint) { if (typeof input !== "object" || input === null) return input; var prim = input[Symbol.toPrimitive]; if (prim !== undefined) { var res = prim.call(input, hint || "default"); if (typeof res !== "object") return res; throw new TypeError("@@toPrimitive must return a primitive value."); } return (hint === "string" ? String : Number)(input); }
var PlusIcon = function PlusIcon(props) {
  return /*#__PURE__*/React.createElement("svg", props, /*#__PURE__*/React.createElement("path", {
    d: "M19 13h-6v6h-2v-6H5v-2h6V5h2v6h6v2z"
  }));
};
PlusIcon.defaultProps = {
  xmlns: "http://www.w3.org/2000/svg",
  width: "24",
  height: "24",
  viewBox: "0 0 24 24"
}; // Libs
// Misc
const defaultMoneyTranslations = _objectSpread(_objectSpread({}, _BaseInput.defaultBaseTranslations), {}, {
  cannotRemoveDefaultCurrency: 'Cannot remove default currency',
  removePrice: 'Remove price',
  addPrice: 'Add price',
  addNewCurrency: 'Add new currency'
});
exports.defaultMoneyTranslations = defaultMoneyTranslations;
class MoneyRaw extends _BaseInput.BaseInput {
  constructor(props) {
    super(props, false);
    this.onCurrencyChanged = (index, currentCurrencies) => e => {
      let newPrices = [].concat(this.props.prices);
      newPrices[index].currency = currentCurrencies.filter(cc => cc.value.toString() === e.target.value)[0];
      this.props.onPricesChange(newPrices);
    };
    this.onTheFlightValidate = value => {
      let num = Number(value);
      const parts = value.split('.');
      if (parts && parts.length > 1 && parts[parts.length - 1].length > 2) {
        return false;
      }
      if (!isNaN(num)) {
        return true;
      }
      if (num) {
        return false;
      }
    };
    this.onPriceChanged = index => e => {
      let newPrices = [].concat(this.props.prices);
      let str = e.target.value;
      let num = Number(str);
      if (!isNaN(num)) {
        newPrices[index].value = num;
      }
      this.props.onPricesChange(newPrices);
      if (!this.state.touched) {
        this.touch();
      }
    };
    this.removePriceClick = index => () => {
      const newPrices = this.props.prices.filter((price, itemIndex) => itemIndex !== index);
      this.props.onPricesChange(newPrices);
      if (newPrices.length === 0 && this.props.required) {
        this.setInvalid([this.getTranslations(defaultMoneyTranslations).required]);
      }
      if (!this.state.touched) {
        this.touch();
      }
    };
    this.addPriceClick = unusedCurrencies => () => {
      this.props.onPricesChange(this.props.prices.concat([{
        value: 0,
        currency: unusedCurrencies[0]
      }]));
      // we are going to focus the last input in the this.containerRef element after a timeout
      setTimeout(() => {
        var _this$containerRef$cu;
        const inputs = (_this$containerRef$cu = this.containerRef.current) === null || _this$containerRef$cu === void 0 ? void 0 : _this$containerRef$cu.querySelectorAll('input');
        if (inputs !== null && inputs !== void 0 && inputs.length) {
          inputs[inputs.length - 1].focus();
        }
      }, 0);
      this.setValid();
      if (!this.state.touched) {
        this.touch();
      }
    };
    this.onFocus = () => this.setState({
      focused: true
    });
    this.onBlur = () => this.setState({
      focused: false
    });
    if (!props.currencies || !props.currencies.length) {
      throw Error('No currencies supplied to money input');
    }
    this.state = Object.assign(this.state, {
      isValid: props.required ? props.prices.length > 0 : true,
      errors: props.required && props.prices.length === 0 ? [this.getTranslations(defaultMoneyTranslations).required] : [],
      handleValueChangeEnabled: false
    });
    const forcedCurrencies = props.currencies.filter(c => c.forceSelected);
    const missingForced = forcedCurrencies.filter(c => !props.prices.find(p => p.currency.value === c.value));
    if (missingForced.length > 0) {
      props.onPricesChange(props.prices.concat(missingForced.map(c => ({
        currency: c,
        value: undefined
      }))));
    }
    this.subscribeSelf(props);
  }
  render() {
    var _Button$defaultProps2;
    let unusedCurrencies = this.props.currencies;
    const translations = this.getTranslations(defaultMoneyTranslations);
    return /*#__PURE__*/React.createElement(_InputGroup.default, {
      title: this.props.title,
      tooltip: this.props.tooltip,
      className: this.props.inputGroupClassName
    }, /*#__PURE__*/React.createElement("div", _extends({}, this.props.id && {
      id: this.props.id
    }, {
      className: 'input__base money-input ' + this.getValidationClass() + (this.props.className ? ' ' + this.props.className : ''),
      ref: this.containerRef
    }), this.props.prices && this.props.prices.map((item, index) => {
      var _currentCurrencies$, _Button$defaultProps;
      let currentCurrencies = this.props.currencies.filter(c => this.props.prices.filter((priceCurrency, priceIndex) => priceIndex !== index && priceCurrency.currency.value === c.value).length === 0);
      const currency = this.props.currencies.find(c => c.value === item.currency.value);
      let retComponents = currentCurrencies.length ? /*#__PURE__*/React.createElement("div", {
        key: index
      }, currentCurrencies.length > 1 ? /*#__PURE__*/React.createElement(_Select.Select, _extends({}, this.props.id && {
        id: this.props.id + '-currency-select-' + index.toString()
      }, {
        onFocus: this.onFocus,
        onBlur: this.onBlur,
        className: 'money-input__select m-0',
        values: currentCurrencies,
        onChange: this.onCurrencyChanged(index, currentCurrencies),
        value: item.currency.value.toString()
      })) : /*#__PURE__*/React.createElement("span", {
        className: "money-input__currencyLabel"
      }, currentCurrencies === null || currentCurrencies === void 0 ? void 0 : (_currentCurrencies$ = currentCurrencies[0]) === null || _currentCurrencies$ === void 0 ? void 0 : _currentCurrencies$.label), /*#__PURE__*/React.createElement(_Text.Text, _extends({}, this.props.id && {
        id: this.props.id + '-amount-input-' + index.toString()
      }, {
        onFocus: this.onFocus,
        onBlur: this.onBlur,
        onTheFlightValidate: this.onTheFlightValidate,
        placeholder: '0.00',
        className: 'money-input__text m-0',
        validators: ['number'],
        value: item.value ? item.value.toString() : '',
        onChange: this.onPriceChanged(index),
        type: "number"
      })), !this.props.disableDelete && this.props.prices.length > 0 && /*#__PURE__*/React.createElement(_Button.Button, _extends({}, this.props.id && {
        id: this.props.id + '-remove-button-' + index.toString()
      }, {
        blank: true,
        type: "error",
        onClick: this.removePriceClick(index),
        className: "transform-rotate--45 line-height--0",
        buttonProps: _objectSpread(_objectSpread({}, (_Button$defaultProps = _Button.Button.defaultProps) === null || _Button$defaultProps === void 0 ? void 0 : _Button$defaultProps.buttonProps), {}, {
          title: currency && currency.forceSelected ? translations.cannotRemoveDefaultCurrency : translations.removePrice
        }),
        circular: true,
        disabled: currency && currency.forceSelected
      }), /*#__PURE__*/React.createElement(PlusIcon, null))) : null;
      unusedCurrencies = unusedCurrencies.filter(c => c.value !== item.currency.value);
      return retComponents;
    }), (this.props.allowMultiple || this.props.prices && !this.props.prices.length) && unusedCurrencies.length ? /*#__PURE__*/React.createElement(_Button.Button, _extends({
      blank: true
    }, this.props.id && {
      id: this.props.id + '-add-button'
    }, {
      type: "primary",
      className: "line-height--0 align-self-start",
      onClick: this.addPriceClick(unusedCurrencies),
      circular: true,
      buttonProps: _objectSpread(_objectSpread({}, (_Button$defaultProps2 = _Button.Button.defaultProps) === null || _Button$defaultProps2 === void 0 ? void 0 : _Button$defaultProps2.buttonProps), {}, {
        title: this.props.prices && this.props.prices.length === 0 ? translations.addPrice : translations.addNewCurrency
      })
    }), /*#__PURE__*/React.createElement(PlusIcon, null)) : null, /*#__PURE__*/React.createElement("span", {
      className: "highlight"
    }), /*#__PURE__*/React.createElement("span", {
      className: 'bar ' + (this.state.focused ? 'focused' : '')
    }), this.renderDefaultValidation(), this.props.label && /*#__PURE__*/React.createElement("label", {
      className: this.props.prices && this.props.prices ? 'label--focused' : ''
    }, this.renderLabel())));
  }
}
exports.MoneyRaw = MoneyRaw;
MoneyRaw.defaultProps = Object.assign({}, _BaseInput.BaseInput.defaultProps, {
  type: 'money',
  allowMultiple: false,
  onChange: undefined
});
const Money = (0, _withThemeContext.withThemeContext)((0, _withFormContext.withFormContext)(MoneyRaw), 'money');
exports.Money = Money;
var _default = Money;
exports.default = _default;
//# sourceMappingURL=Money.js.map