"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = exports.NumberInputRaw = exports.NumberInput = void 0;
var React = _interopRequireWildcard(require("react"));
var _InputGroup = _interopRequireDefault(require("../inputGroup/InputGroup"));
var _BaseInput = require("../base/input/BaseInput");
var _withFormContext = require("../form/withFormContext");
var _withThemeContext = require("../themeProvider/withThemeContext");
var _classnames = _interopRequireDefault(require("classnames"));
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }
function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }
function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }
function _extends() { _extends = Object.assign ? Object.assign.bind() : function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }
var ArrowIcon = function ArrowIcon(props) {
  return /*#__PURE__*/React.createElement("svg", props, /*#__PURE__*/React.createElement("path", {
    d: "M7.41 7.84L12 12.42l4.59-4.58L18 9.25l-6 6-6-6z"
  }));
};
ArrowIcon.defaultProps = {
  xmlns: "http://www.w3.org/2000/svg",
  width: "24",
  height: "24",
  viewBox: "0 0 24 24"
}; // Libs
// Misc
class NumberInputRaw extends _BaseInput.BaseInput {
  constructor(props) {
    super(props);
    this.onBlur = () => {
      this.commitState();
      this.handleBlur();
    };
    this.removeUnitClick = e => {
      e.preventDefault();
      this.handleLimits((this.props.number || 0) - 1);
    };
    this.onUnitChanged = e => {
      this.setState({
        currentText: e.target.value
      });
    };
    this.onKeyDown = e => {
      if (e.key === 'Enter') {
        this.commitState();
      }
    };
    this.addUnitClick = e => {
      e.preventDefault();
      this.handleLimits((this.props.number || 0) + 1);
    };
    this.removeUnitClick = this.removeUnitClick.bind(this);
    this.addUnitClick = this.addUnitClick.bind(this);
    this.onUnitChanged = this.onUnitChanged.bind(this);
  }
  render() {
    const {
      unit
    } = this.props;
    return /*#__PURE__*/React.createElement(_InputGroup.default, {
      title: this.props.title,
      className: this.props.inputGroupClassName,
      tooltip: this.props.tooltip
    }, /*#__PURE__*/React.createElement("div", _extends({}, this.props.id && {
      id: this.props.id
    }, {
      className: 'input__base number-input ' + this.getValidationClass() + ' ' + (this.props.className ? this.props.className : ''),
      ref: this.containerRef
    }), /*#__PURE__*/React.createElement("div", {
      className: ""
    }, /*#__PURE__*/React.createElement("div", {
      className: "number-input__arrows__container"
    }, /*#__PURE__*/React.createElement("button", _extends({
      className: "plus",
      onClick: this.addUnitClick
    }, this.props.id && {
      id: this.props.id + '-add-hours-button'
    }, {
      tabIndex: 0
    }), /*#__PURE__*/React.createElement(ArrowIcon, null)), /*#__PURE__*/React.createElement("div", {
      className: "input-padding"
    }, /*#__PURE__*/React.createElement("input", _extends({}, this.props.id && {
      id: this.props.id + '-input'
    }, {
      disabled: this.getDisabled(),
      required: this.props.required,
      className: (0, _classnames.default)('number-input__time', {
        filled: this.props.number !== undefined
      }),
      onChange: this.onUnitChanged,
      value: this.state.currentText !== undefined ? this.state.currentText : this.props.number,
      onBlur: this.onBlur,
      onFocus: this.handleFocus,
      type: "number",
      onKeyDown: this.onKeyDown
    })), /*#__PURE__*/React.createElement("span", {
      className: "highlight"
    })), /*#__PURE__*/React.createElement("button", _extends({
      className: "minus",
      onClick: this.removeUnitClick
    }, this.props.id && {
      id: this.props.id + '-subtract-hours-button'
    }, {
      tabIndex: 0
    }), /*#__PURE__*/React.createElement(ArrowIcon, null))), unit), this.renderDefaultValidation(), /*#__PURE__*/React.createElement("span", {
      className: "highlight"
    }), /*#__PURE__*/React.createElement("span", {
      className: 'bar ' + (this.state.focused ? 'focused' : '')
    }), this.props.label && /*#__PURE__*/React.createElement("label", {
      className: (0, _classnames.default)('number-input__label label--focused')
    }, this.renderLabel())));
  }
  commitState() {
    if (this.state.currentText) {
      var num = Number(this.state.currentText);
      if (!isNaN(num)) {
        this.handleLimits(num);
      }
      this.setState({
        currentText: undefined
      });
    }
  }
  handleLimits(num) {
    if (this.props.min !== undefined) {
      if (num - this.props.min <= 0) {
        this.props.onNumberChange(this.props.min);
        return;
      }
    }
    if (this.props.max !== undefined) {
      if (this.props.max - num <= 0) {
        this.props.onNumberChange(this.props.max);
        return;
      }
    }
    this.props.onNumberChange(num);
  }
}
exports.NumberInputRaw = NumberInputRaw;
NumberInputRaw.defaultProps = Object.assign({}, _BaseInput.BaseInput.defaultProps, {
  type: 'time',
  placeholder: ''
});
const NumberInput = (0, _withThemeContext.withThemeContext)((0, _withFormContext.withFormContext)(NumberInputRaw), 'numberInput');
exports.NumberInput = NumberInput;
var _default = NumberInput;
exports.default = _default;
//# sourceMappingURL=NumberInput.js.map