import * as React from 'react';
import { BaseInputProps, BaseInputState, BaseInput, ValidationError } from '../base/input/BaseInput';
export interface SelectValue {
    value: number | string;
    label?: string;
    forceSelected?: boolean;
}
export interface SelectProps extends BaseInputProps<HTMLSelectElement> {
    values?: SelectValue[];
    defaultEmpty?: boolean;
    multiple?: boolean;
    selectedValues?: SelectValue[];
    onSelectedValuesChange?: (newValues: SelectValue[]) => void;
    inputRef?: (input: HTMLSelectElement) => void;
    readOnly?: boolean;
    readonlyEmptyPlaceholder?: string;
}
export interface SelectState extends BaseInputState {
}
export declare class SelectRaw extends BaseInput<SelectProps, SelectState, HTMLSelectElement> {
    static defaultProps: {
        theme?: import("../../types/Theme").Theme;
        id?: string;
        disabled?: boolean;
        className?: string;
        label?: string | JSX.Element;
        tooltip?: string | JSX.Element;
        tooltipProps?: import("@tippy.js/react").TippyProps;
        value?: string;
        onChange?: (e: React.ChangeEvent<never>, isValid: boolean) => void;
        required?: boolean;
        customValidators?: import("../..").IBaseValidator[];
        validators?: import("../..").ValidatorTypes[];
        noValidate?: boolean;
        validationName?: string;
        touchOn?: "focus" | "blur";
        ignoreContext?: boolean;
        onTheFlightValidate?: (value: string) => boolean;
        onFocus?: () => void;
        onBlur?: () => void;
        onSuggestionsClosed?: () => void;
        onSuggestionsOpened?: () => void;
        onClick?: (e: React.MouseEvent<Element, MouseEvent>) => void;
        title?: string | JSX.Element;
        onKeyDown?: (e: React.KeyboardEvent<HTMLElement>) => void;
        errors?: ValidationError[];
        onErrorsChanged?: (errors: ValidationError[]) => void;
        showValidation?: boolean;
        reRendersWhenContextChanges?: boolean;
        defaultTouched?: boolean;
        translations?: Partial<{
            required: string;
        }>;
        inputGroupClassName?: string;
        infoText?: string | JSX.Element;
    } & import("../form").FormContextProps & {
        defaultEmpty: boolean;
        multiple: boolean;
        readOnly: boolean;
        readonlyEmptyPlaceholder: string;
    };
    constructor(props: SelectProps);
    componentDidMount(): void;
    componentDidUpdate(oldProps: SelectProps): void;
    render(): React.JSX.Element;
    protected handleValueChange(value: string, isValid?: boolean, errors?: ValidationError[], props?: SelectProps, initializing?: boolean): {
        isValid: boolean;
        errors: ValidationError[];
    };
    protected handleBlur(): void;
    protected handleFocus(): void;
    private handleChangeCustom;
    private handleValid;
    private renderReadonly;
    private renderSelectedValues;
    private removeItemClick;
}
export declare const Select: React.ForwardRefExoticComponent<SelectProps & React.RefAttributes<SelectRaw>>;
export default Select;
