import * as React from 'react';
import { BaseInputProps, BaseInputState, BaseInput } from '../base/input/BaseInput';
import { ButtonProps } from '../button/Button';
export declare const defaultSubmitTranslations: {
    hangOn: string;
    needsFixing: string;
    required: string;
};
export type SubmitTranslations = Partial<typeof defaultSubmitTranslations>;
export type SubmitProps = BaseInputProps<never, SubmitTranslations> & ButtonProps & {
    onClick?: (e: React.MouseEvent<HTMLButtonElement>) => void;
    validateForm?: boolean;
    disabledTitle?: string;
    showValidationSummaryTooltip?: boolean;
};
export interface SubmitState extends BaseInputState {
}
export declare class SubmitRaw extends BaseInput<SubmitProps, SubmitState, never, SubmitTranslations> {
    static defaultProps: {
        theme?: import("../../types/Theme").Theme;
        id?: string;
        disabled?: boolean;
        className?: string;
        label?: string | JSX.Element;
        tooltip?: string | JSX.Element;
        tooltipProps?: import("@tippy.js/react").TippyProps;
        value?: string;
        onChange?: (e: React.ChangeEvent<never>, isValid: boolean) => void;
        required?: boolean;
        customValidators?: import("../..").IBaseValidator[];
        validators?: import("../..").ValidatorTypes[];
        noValidate?: boolean;
        validationName?: string;
        touchOn?: "focus" | "blur";
        ignoreContext?: boolean;
        onTheFlightValidate?: (value: string) => boolean;
        onFocus?: () => void;
        onBlur?: () => void;
        onSuggestionsClosed?: () => void;
        onSuggestionsOpened?: () => void;
        onClick?: (e: React.MouseEvent<Element, MouseEvent>) => void;
        title?: string | JSX.Element;
        onKeyDown?: (e: React.KeyboardEvent<HTMLElement>) => void;
        errors?: import("../base/input/BaseInput").ValidationError[];
        onErrorsChanged?: (errors: import("../base/input/BaseInput").ValidationError[]) => void;
        showValidation?: boolean;
        reRendersWhenContextChanges?: boolean;
        defaultTouched?: boolean;
        translations?: Partial<{
            required: string;
        }>;
        inputGroupClassName?: string;
        infoText?: string | JSX.Element;
    } & import("../form").FormContextProps & {
        validateForm: boolean;
        ignoreContext: boolean;
        reRendersWhenContextChanges: boolean;
        showValidationSummaryTooltip: boolean;
        preventsDefault: boolean;
    };
    constructor(props: SubmitProps);
    render(): React.JSX.Element;
    private handleClick;
    private isDisabled;
}
export declare const Submit: React.ForwardRefExoticComponent<{
    theme?: import("../../types/Theme").Theme;
    id?: string;
    disabled?: boolean;
    className?: string;
    label?: string | JSX.Element;
    tooltip?: string | JSX.Element;
    tooltipProps?: import("@tippy.js/react").TippyProps;
    value?: string;
    onChange?: (e: React.ChangeEvent<never>, isValid: boolean) => void;
    required?: boolean;
    customValidators?: import("../..").IBaseValidator[];
    validators?: import("../..").ValidatorTypes[];
    noValidate?: boolean;
    validationName?: string;
    touchOn?: "focus" | "blur";
    ignoreContext?: boolean;
    onTheFlightValidate?: (value: string) => boolean;
    onFocus?: () => void;
    onBlur?: () => void;
    onSuggestionsClosed?: () => void;
    onSuggestionsOpened?: () => void;
    onClick?: (e: React.MouseEvent<Element, MouseEvent>) => void;
    title?: string | JSX.Element;
    onKeyDown?: (e: React.KeyboardEvent<HTMLElement>) => void;
    errors?: import("../base/input/BaseInput").ValidationError[];
    onErrorsChanged?: (errors: import("../base/input/BaseInput").ValidationError[]) => void;
    showValidation?: boolean;
    reRendersWhenContextChanges?: boolean;
    defaultTouched?: boolean;
    translations?: Partial<{
        hangOn: string;
        needsFixing: string;
        required: string;
    }>;
    inputGroupClassName?: string;
    infoText?: string | JSX.Element;
} & import("../form").FormContextProps & {
    id?: string;
    onClick?: (e: React.MouseEvent<HTMLButtonElement, MouseEvent>) => void;
    style?: React.CSSProperties;
    className?: string;
    disabled?: boolean;
    type?: import("../button/Button").ButtonTypes;
    circular?: boolean;
    noRipples?: boolean;
    small?: boolean;
    icon?: boolean;
    outlined?: boolean;
    disableAfterClickMs?: number;
    disableAfterClick?: boolean;
    buttonProps?: React.ButtonHTMLAttributes<HTMLButtonElement>;
    text?: boolean;
    hero?: boolean;
    noShadow?: boolean;
    blank?: boolean;
    unobtrusive?: boolean;
    dropdown?: boolean;
    Component?: React.FC<import("../button/Button").ButtonComponentProps>;
    tooltip?: string | JSX.Element;
    disableTooltip?: boolean;
    tooltipProps?: import("@tippy.js/react").TippyProps;
    preventsDefault?: boolean;
} & import("../themeProvider").ThemeContextProps & {
    children?: React.ReactNode;
} & {
    onClick?: (e: React.MouseEvent<HTMLButtonElement>) => void;
    validateForm?: boolean;
    disabledTitle?: string;
    showValidationSummaryTooltip?: boolean;
} & React.RefAttributes<SubmitRaw>>;
export default Submit;
