import * as React from 'react';
import { TextProps } from '../text/Text';
import { BaseInputProps, BaseInputState, BaseInput, ValidationError } from '../base/input/BaseInput';
import { RenderSuggestionTagProps } from './subComponents/TagsSuggestions';
import { PopperProps } from '@mui/material/Popper/Popper';
export type Tag<T extends number | string = number> = {
    id: T;
    name: string;
};
export type ResourceTConstraint<IdT extends number | string> = {
    id: IdT;
};
export declare const defaultTagsTranslations: {
    addNew: string;
    valueNotAddedError: string;
    maxTagsSurpassedError: string;
    waitingForMoreInputComponent: string;
    suggestionsEmptyComponent: string;
    suggestionsEmptyAllowNewComponent: string;
    required: string;
};
export type TagsTranslations = Partial<typeof defaultTagsTranslations>;
export type TagsProps<IdT extends number | string = number, T extends ResourceTConstraint<IdT> = Tag<IdT>> = {
    className?: string;
    tagsSuggestionsClassName?: string;
    disabled?: boolean;
    tags: T[];
    existingTags?: T[];
    fetchExistingTags?: (text: string) => Promise<T[]>;
    onTagsChanged: (newTags: T[]) => void;
    onNewTagAdded?: (newTagName: string) => Promise<T>;
    onTagClick?: (tag: T) => void;
    showChips?: boolean;
    allowNew?: boolean;
    addNewOnBlur?: boolean;
    textProps?: TextProps;
    readOnly?: boolean;
    readonlyEmptyPlaceholder?: string;
    maxTags?: number;
    valueNotAddedError?: string | JSX.Element;
    maxTagsSurpassedError?: string | JSX.Element;
    showSuggestions?: boolean;
    suggestionsLoadingComponent?: string | JSX.Element;
    suggestionsEmptyComponent?: string | JSX.Element | null;
    suggestionsEmptyAllowNewComponent?: string | JSX.Element | null;
    waitingForMoreInputComponent?: string | JSX.Element;
    loadingDelayMs?: number;
    filterExistingTags?: (text: string, existingTags: T[]) => T[];
    allowSameTagMultipleTimes?: boolean;
    maxSuggestions?: number;
    popperProps?: Partial<PopperProps>;
    minLettersToFetch?: number;
    mobileVersionEnabled?: boolean;
    isLoading?: boolean;
    closeSuggestionsAfterCreate?: boolean;
    SuggestionTag?: React.ComponentType<RenderSuggestionTagProps<IdT, T>>;
    getName?: (tag: T) => string;
} & BaseInputProps<HTMLInputElement, TagsTranslations>;
export interface TagsState<IdT extends number | string = number, T extends ResourceTConstraint<IdT> = Tag<IdT>> extends BaseInputState {
    textIsFocused: boolean;
    textErrors: ValidationError[];
    textIsValid: boolean;
    suggestionsVisible: boolean;
    fetchedExistingTags: T[];
    fetchingExistingTags: boolean;
    preselectedSuggestion?: number;
}
type InjectedProps = {};
export declare class TagsRaw<IdT extends number | string = number, T extends ResourceTConstraint<IdT> = Tag<IdT>> extends BaseInput<TagsProps<IdT, T> & InjectedProps, TagsState<IdT, T>, HTMLInputElement, TagsTranslations> {
    static defaultProps: TagsProps;
    private textRef;
    private suggestionsRef;
    private isMobile;
    constructor(props: TagsProps<IdT, T> & InjectedProps);
    focus(): void;
    componentDidUpdate(oldProps: TagsProps<IdT, T> & InjectedProps): void;
    handleLeaveMobileClick(): void;
    handleClickOutside(e: MouseEvent): void;
    componentDidMount(): void;
    componentWillUnmount(): void;
    render(): React.JSX.Element;
    private onTextErrorsChanged;
    private onFocus;
    private onKeyDown;
    private addNewTag;
    private onSuggestionSelected;
    private onBlur;
    private onTextChanged;
    private handleErrors;
    private getErrors;
    private fetchExistingTags;
    private getSuggestions;
    private renderTag;
    private tagClick;
    private tagRemoveClick;
}
interface TagsFinal {
    <IdT extends number | string = number, T extends ResourceTConstraint<IdT> = Tag<IdT>>(item: TagsProps<IdT, T>): React.ReactElement;
    defaultProps?: Partial<TagsProps<any, any>>;
}
export declare const Tags: TagsFinal;
export default Tags;
