"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = exports.allTimeSpanUnits = exports.TimeSpanUnit = exports.TimeSpanRaw = exports.TimeSpan = void 0;
var React = _interopRequireWildcard(require("react"));
var _InputGroup = _interopRequireDefault(require("../inputGroup/InputGroup"));
var _BaseInput = require("../base/input/BaseInput");
var _withFormContext = require("../form/withFormContext");
var _moment = _interopRequireDefault(require("moment"));
var _withThemeContext = require("../themeProvider/withThemeContext");
var _classnames = _interopRequireDefault(require("classnames"));
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }
function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }
function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }
function _extends() { _extends = Object.assign ? Object.assign.bind() : function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }
var ArrowIcon = function ArrowIcon(props) {
  return /*#__PURE__*/React.createElement("svg", props, /*#__PURE__*/React.createElement("path", {
    d: "M7.41 7.84L12 12.42l4.59-4.58L18 9.25l-6 6-6-6z"
  }));
};
ArrowIcon.defaultProps = {
  xmlns: "http://www.w3.org/2000/svg",
  width: "24",
  height: "24",
  viewBox: "0 0 24 24"
}; // Libs
// Misc
let TimeSpanUnit = /*#__PURE__*/function (TimeSpanUnit) {
  TimeSpanUnit[TimeSpanUnit["Dynamic"] = 0] = "Dynamic";
  TimeSpanUnit[TimeSpanUnit["Millisecond"] = 1] = "Millisecond";
  TimeSpanUnit[TimeSpanUnit["Second"] = 2] = "Second";
  TimeSpanUnit[TimeSpanUnit["Minute"] = 4] = "Minute";
  TimeSpanUnit[TimeSpanUnit["Hour"] = 8] = "Hour";
  TimeSpanUnit[TimeSpanUnit["Day"] = 16] = "Day";
  TimeSpanUnit[TimeSpanUnit["Year"] = 32] = "Year";
  return TimeSpanUnit;
}({});
exports.TimeSpanUnit = TimeSpanUnit;
const allTimeSpanUnits = TimeSpanUnit.Day | TimeSpanUnit.Hour | TimeSpanUnit.Millisecond | TimeSpanUnit.Minute | TimeSpanUnit.Second | TimeSpanUnit.Year;
exports.allTimeSpanUnits = allTimeSpanUnits;
function getValues(enumeration) {
  var values = [];
  for (var value in enumeration) {
    if (isNaN(Number(value))) {
      values.push(enumeration[value]);
    }
  }
  return values;
}
const allUnits = getValues(TimeSpanUnit);
class TimeSpanRaw extends _BaseInput.BaseInput {
  constructor(props) {
    super(props);
    this.onBlur = () => {
      this.commitState();
      this.handleBlur();
    };
    this.removeUnitClick = e => {
      var _this$props$timeSpan;
      e.preventDefault();
      const unit = Number(e.currentTarget.dataset.unit);
      const diff = -1 * this.oneUnitInMs(unit);
      const newDate = _moment.default.duration((((_this$props$timeSpan = this.props.timeSpan) === null || _this$props$timeSpan === void 0 ? void 0 : _this$props$timeSpan.asMilliseconds()) || 0) + diff, 'millisecond');
      this.handleLimits(newDate);
    };
    this.onUnitChanged = e => {
      // this.handleHoursChange(e.target.value);
      const unit = Number(e.currentTarget.dataset.unit);
      this.setState({
        currentUnitText: e.target.value,
        currentUnit: unit
      });
    };
    this.onKeyDown = e => {
      if (e.key === 'Enter') {
        this.commitState();
      }
    };
    this.addUnitClick = e => {
      var _this$props$timeSpan2;
      e.preventDefault();
      const unit = Number(e.currentTarget.dataset.unit);
      const diff = this.oneUnitInMs(unit);
      const newDate = _moment.default.duration((((_this$props$timeSpan2 = this.props.timeSpan) === null || _this$props$timeSpan2 === void 0 ? void 0 : _this$props$timeSpan2.asMilliseconds()) || 0) + diff, 'milliseconds');
      this.handleLimits(newDate);
    };
    this.removeUnitClick = this.removeUnitClick.bind(this);
    this.addUnitClick = this.addUnitClick.bind(this);
    this.onUnitChanged = this.onUnitChanged.bind(this);
  }
  render() {
    const {
      timeSpan,
      validUnits,
      initialUnits,
      units: _units
    } = this.props;
    const units = _units || this.getUnits(timeSpan, validUnits, initialUnits);
    const unitsArr = allUnits.filter(u => (units & u) !== 0).reverse();
    return /*#__PURE__*/React.createElement(_InputGroup.default, {
      title: this.props.title,
      tooltip: this.props.tooltip,
      className: this.props.inputGroupClassName
    }, /*#__PURE__*/React.createElement("div", _extends({}, this.props.id && {
      id: this.props.id
    }, {
      className: 'input__base timeSpan-input ' + this.getValidationClass() + ' ' + (this.props.className ? this.props.className : ''),
      ref: this.containerRef
    }), unitsArr.map(unit => /*#__PURE__*/React.createElement("div", {
      className: "",
      key: unit
    }, /*#__PURE__*/React.createElement("div", {
      className: "timeSpan-input__arrows__container"
    }, /*#__PURE__*/React.createElement("button", _extends({
      className: "plus",
      onClick: this.addUnitClick
    }, this.props.id && {
      id: this.props.id + '-add-hours-button'
    }, {
      "data-unit": unit,
      tabIndex: 0
    }), /*#__PURE__*/React.createElement(ArrowIcon, null)), /*#__PURE__*/React.createElement("div", {
      className: "input-padding"
    }, /*#__PURE__*/React.createElement("input", _extends({}, this.props.id && {
      id: this.props.id + '-' + unit + '-input'
    }, {
      disabled: this.getDisabled(),
      required: this.props.required,
      className: (0, _classnames.default)('timeSpan-input__time', {
        filled: this.state.value,
        'timeSpan-input__time--ms': unit === TimeSpanUnit.Millisecond
      }),
      onChange: this.onUnitChanged,
      value: this.state.currentUnit === unit ? this.state.currentUnitText : this.getValue(timeSpan, unit),
      onBlur: this.onBlur,
      onFocus: this.handleFocus,
      type: "number",
      "data-unit": unit,
      onKeyDown: this.onKeyDown
    })), /*#__PURE__*/React.createElement("span", {
      className: "highlight"
    })), /*#__PURE__*/React.createElement("button", _extends({
      className: "minus",
      onClick: this.removeUnitClick
    }, this.props.id && {
      id: this.props.id + '-subtract-hours-button'
    }, {
      "data-unit": unit,
      tabIndex: 0
    }), /*#__PURE__*/React.createElement(ArrowIcon, null))), this.props.unitToString(unit))), this.renderDefaultValidation(), /*#__PURE__*/React.createElement("span", {
      className: "highlight"
    }), /*#__PURE__*/React.createElement("span", {
      className: 'bar ' + (this.state.focused ? 'focused' : '')
    }), this.props.label && /*#__PURE__*/React.createElement("label", {
      className: (0, _classnames.default)('timeSpan-input__label label--focused')
    }, this.renderLabel())));
  }
  getUnits(time, validUnits, initialUnits) {
    if (time === null || time === undefined) {
      return initialUnits;
    }
    const ms = time.asMilliseconds();
    let units = TimeSpanUnit.Millisecond | TimeSpanUnit.Second;
    if (ms >= 1000) {
      units += TimeSpanUnit.Minute;
    }
    if (ms >= 60000) {
      units += TimeSpanUnit.Hour;
    }
    if (ms >= 60 * 60000) {
      units += TimeSpanUnit.Day;
    }
    if (ms >= 24 * 60 * 60000) {
      units += TimeSpanUnit.Year;
    }
    const tagMask = allTimeSpanUnits & ~validUnits;
    units &= ~tagMask;
    return units || initialUnits;
  }
  getValue(time, unit) {
    if (!time || !unit) {
      return 0;
    }
    switch (unit) {
      case TimeSpanUnit.Millisecond:
        return Math.floor(time.get('millisecond'));
      case TimeSpanUnit.Second:
        return Math.floor(time.get('second'));
      case TimeSpanUnit.Minute:
        return Math.floor(time.get('minute'));
      case TimeSpanUnit.Hour:
        return Math.floor(time.get('hour'));
      case TimeSpanUnit.Day:
        return Math.floor(time.get('day'));
      case TimeSpanUnit.Year:
        return Math.floor(time.get('year'));
      default:
        return 0;
    }
  }
  oneUnitInMs(unit) {
    if (!unit) {
      return 0;
    }
    switch (unit) {
      case TimeSpanUnit.Millisecond:
        return 1;
      case TimeSpanUnit.Second:
        return 1000;
      case TimeSpanUnit.Minute:
        return 60000;
      case TimeSpanUnit.Hour:
        return 3600000;
      case TimeSpanUnit.Day:
        return 24 * 3600000;
      case TimeSpanUnit.Year:
        return 24 * 365 * 3600000;
      default:
        return 0;
    }
  }
  maxOfUnit(unit) {
    if (!unit) {
      return 0;
    }
    switch (unit) {
      case TimeSpanUnit.Millisecond:
        return 1000;
      case TimeSpanUnit.Second:
        return 60;
      case TimeSpanUnit.Minute:
        return 60;
      case TimeSpanUnit.Hour:
        return 24;
      case TimeSpanUnit.Day:
        return 365;
      case TimeSpanUnit.Year:
        return 9999;
      default:
        return 0;
    }
  }
  commitState() {
    if (this.state.currentUnit) {
      var num = Number(this.state.currentUnitText);
      if (!isNaN(num)) {
        const newOfUnitMs = num * this.oneUnitInMs(this.state.currentUnit);
        const oldOfUnitMs = this.getValue(this.props.timeSpan, this.state.currentUnit) * this.oneUnitInMs(this.state.currentUnit);
        const maxOfUnitMs = this.maxOfUnit(this.state.currentUnit) * this.oneUnitInMs(this.state.currentUnit);
        const moreThanMax = maxOfUnitMs < newOfUnitMs;
        const diff = newOfUnitMs - oldOfUnitMs;
        if (moreThanMax) {
          this.handleLimits(_moment.default.duration(newOfUnitMs, 'millisecond'));
        } else {
          var _this$props$timeSpan3;
          this.handleLimits(_moment.default.duration((((_this$props$timeSpan3 = this.props.timeSpan) === null || _this$props$timeSpan3 === void 0 ? void 0 : _this$props$timeSpan3.asMilliseconds()) || 0) + diff, 'millisecond'));
        }
      }
      this.setState({
        currentUnit: undefined,
        currentUnitText: undefined
      });
    }
  }
  handleLimits(time) {
    if (this.props.min) {
      if (time.asMilliseconds() - this.props.min.asMilliseconds() <= 0) {
        this.props.timeSpanChange(this.props.min.clone());
        return;
      }
    }
    if (this.props.max) {
      if (this.props.max.asMilliseconds() - time.asMilliseconds() <= 0) {
        this.props.timeSpanChange(this.props.max.clone());
        return;
      }
    }
    this.props.timeSpanChange(time);
  }
}
exports.TimeSpanRaw = TimeSpanRaw;
TimeSpanRaw.defaultProps = Object.assign({}, _BaseInput.BaseInput.defaultProps, {
  type: 'time',
  placeholder: '',
  validUnits: allTimeSpanUnits,
  initialUnits: allTimeSpanUnits,
  units: TimeSpanUnit.Dynamic,
  unitToString: unit => {
    switch (unit) {
      case TimeSpanUnit.Millisecond:
        return 'ms';
      case TimeSpanUnit.Second:
        return 's';
      case TimeSpanUnit.Minute:
        return 'm';
      case TimeSpanUnit.Hour:
        return 'h';
      case TimeSpanUnit.Day:
        return 'd';
      case TimeSpanUnit.Year:
        return 'y';
      default:
        return '';
    }
  },
  min: _moment.default.duration(0)
});
const TimeSpan = (0, _withThemeContext.withThemeContext)((0, _withFormContext.withFormContext)(TimeSpanRaw), 'timeSpan');
exports.TimeSpan = TimeSpan;
var _default = TimeSpan;
exports.default = _default;
//# sourceMappingURL=TimeSpan.js.map